package sql::Table;

########## Confixx(R) 3.1 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.swsoft.com - info@swsoft.com ####


use strict;

use sql::Field;

sub new {
  my $classname = shift;
  my $args = {};
  if ( @_ == 1 ) {
    $args->{'fullsql'}=shift;
  } else {
    $args = {'name'=>'',
	     'sql'=>'',
	     'fullsql'=>'',
	      @_
	    };
  }
  my $this = {'name'=>$args->{'name'},
							'sql'=>$args->{'sql'},
							'fullsql'=>$args->{'fullsql'},
							'fields'=>{},
							'fldnames'=>[],
						 };

  bless ($this, $classname);
  return $this;
}

sub Name {
  my $this = shift;
  if ( @_ ) {
    $this->{'name'} = shift;
  }
  return $this->{'name'};
}

sub Correct {
  my $this = shift;
  if ( @_ ) {
    $this->{'correct'} = shift;
  }
  return $this->{'correct'};
}


sub Field {
  my $this = shift;
  my $name = shift;
  my $field;
  if ( exists($this->{'fields'}->{$name}) ) {
    if ( @_ ) {
      $field = shift;
      if ( ref($field ) =~ /Field/ ) {
				$this->{'fields'}->{$name} = $field;
      }
    }
    return $this->{'fields'}->{$name};
  }else{
    return undef;
  }
}

sub Fields {
  my $this = shift;
  if ( @_ ) {
    my $refHash = shift;
    $this->{'fields'} = $refHash if ref($refHash) =~ /HASH/;
  }
  return $this->{'fields'};
}

sub NamesOfFields {
  my $this = shift;
  if ( @_ ) {
    my $refHash = shift;
    $this->{'fldnames'} = $refHash if ref($refHash) =~ /ARRAY/;
  }
  return $this->{'fldnames'};
}

sub appendField {
  my $this = shift;
  my $field = shift;
  if ( ref($field) =~ /Field/ ) {
    my $name = $field->Name;
    if ( exists( $this->{'fields'}->{$name} ) ) {
      $field = undef;
    } else {
      $this->{'fields'}->{$name} = $field;
      push @{$this->{'fldnames'}},$name;
    }
  }
  return $field;
}

sub parseSQL {
  my $this = shift;
  my $sql = shift || $this->{'sql'};
  unless ($sql =~ s/^\s*create\s+table\s+(\S+)\s*\(//i){
    return 0;
  }
  $this->{'fields'} = {};
  $this->{'fldnames'} = [];
  $this->{'indexes'} = [];
  $this->{'unique'} = [];

  $this->{'name'} = $1;
  if ( $::DEBUG>1 ) {
    print STDERR "Start parsing table ".$this->{'name'}."\n";
  }
  my ($fieldSQL,$field,$fieldName,$tail,$index);
  if ( $sql =~ s/\)([^)]+);\s*$/);/ ) {
    $this->{'suffix'} = $1;
    $this->{'suffix'} =~ s/^\s+//;
    $this->{'suffix'} =~ s/\s+$//;
  }
  while ($sql =~ s/((?:'(?>[^']*)(?>['\]'[^']*)*'|"(?>[^"]*)(?>["\]"[^"]*)*"|\([^)(]*\)|[^'",)(])*)(,|\)\s*;)//){
    $fieldSQL = $1;
    if ( $fieldSQL =~ s/^\s*primary\s+key(?=[ (])//i ) {
      if ( $fieldSQL =~ s/\s*\(([^()]+)\)// ) {
				$this->{'id'} = $1;
      } else {
				print STDERR "Table (".$this->{'name'}."): parse error: 'primary key'-statement: $fieldSQL\n";
      }
    } elsif ( $fieldSQL =~ s/^\s*index(?=[ (])//i ) {
      if ( $fieldSQL=~s/\s*\(([^()]+)\)// ) {
				push @{$this->{'indexes'}},$1;
      } else {
				print STDERR "Table (".$this->{'name'}."): parse error: 'index'-statement: $fieldSQL\n";
      }
    } elsif ( $fieldSQL =~ s/^\s*unique(?=[ (])//i ) {
      if ( $fieldSQL =~ s/\s*\(([^()]+)\)// ) {
				push @{$this->{'unique'}},$1;
      } else {
				print STDERR "Table (".$this->{'name'}."): parse error: 'unique'-statement: $fieldSQL\n";
      }
    } elsif ( $fieldSQL =~ s/^\s*key(?=[ (])//i ) {
      $fieldSQL =~ s/^\s+//;
      if ( $fieldSQL =~ s/^(\S+)(?=[ (])// ) {
				$index = $1;
      }
      if ( $fieldSQL =~ s/\s*\(([^()]+)\)// ) {
				$index = $index?$index."($1)":$1;
				push @{$this->{'indexes'}},$index;
      } else {
				print STDERR "Table (".$this->{'name'}."): parse error: 'key'-statement: $fieldSQL\n";
      }
    } else {
      $field = sql::Field->new( $fieldSQL );
      if ( $field->parseSQL() ) {
				$fieldName = $field->Name;
				if ( exists( $this->{'fields'}->{$fieldName} ) ) {
					print STDERR "Table (".$this->{'name'}."): parse error: dublicate field '$fieldName'\n";
				} else{
					$this->{'fields'}->{$fieldName} = $field;
					push @{$this->{'fldnames'}},$fieldName;
					if ( $field->ID ) {
						$this->{'id'} = $fieldName;
					}
				}
      }
    }
  }
  $sql =~ s/^\s+//;
  $sql =~ s/\s+$//;
  if ( $sql ) {
    print STDERR "Table (".$this->{'name'}."): parse error: wrong tail\t=\t'$sql'\n";
    return 0;
  }
  return 1;
}

sub ID {
  my $this = shift;
  if ( @_ ) {
    my $name = shift;
    if ( exists( $this->{'fields'}->{$name} ) ) {
      $this->{'id'}=$name;
    }
  }
  return $this->{'id'};
}

sub SQL {
  my $this = shift;
  if ( @_ ) {
    $this->{'sql'}=shift;
  }
  return $this->{'sql'};
}

sub appendSQL {
  my $this = shift;
  if ( @_ ) {
    $this->{'sql'} .= shift;
  }
  return $this->{'sql'};
}

sub FullSQL {
  my $this = shift;
  if ( @_ ) {
    $this->{'fullsql'} = shift;
  }
  return $this->{'fullsql'};
}

sub appendFullSQL {
  my $this = shift;
  if ( @_ ) {
    $this->{'fullsql'}.=shift;
  }
  return $this->{'fullsql'};
}

sub getSQL {
  my $this = shift;
  my $sql = $this->{'fullsql'};
  while ( chomp $sql ){
    $sql =~ s/--.*$//;  ## clean comment
  }
  $sql =~ s/;\s*$//;
  return $sql;
}

sub fillFromDB {
  my $this = shift;
  my $sth = shift;
  return 0 unless( ref($sth)=~/DBI/ );

#
# clean
#
  $this->{'fields'} = {};
  $this->{'fldnames'} = [];
  $this->{'indexes'} = [];
  $this->{'unique'} = [];
  
  my($fldNumb,$j,$fldName,$fldType,$fldNull,$field);
  my $dbIsPg = ( $::dbType eq 'Pg' );

  $fldNumb = $sth->{NUM_OF_FIELDS};
  for ( $j = 0; $j < $fldNumb; $j++ ) {
    if ($dbIsPg){
      $fldName = lc($sth->{NAME}->[$j]);
    } else {
      $fldName = $sth->{NAME}->[$j];
    }
    $fldType = $sth->{TYPE}->[$j];
    $fldNull = $sth->{NULLABLE}->[$j];
    $field = sql::Field->new('name'=>$fldName);
    $field->NotNull(1) unless $fldNull;
    if ( $dbIsPg ) {
      if ( $fldType == 5 ) {  ## smallint
				$field->Type('smallint');
      } elsif ( $fldType == 4) { ## int
				$field->Type('int');
      } elsif ( $fldType == 8 ) { ## bigint
				$field->Type('bigint');
      } elsif ( $fldType == 2 ) { ## real
				$field->Type('real');
      } elsif ( $fldType == 7 ) { ## double
				$field->Type('double');
      } elsif ( $fldType == 1700 ) { ## decimal
				$field->Type('decimal');
      } elsif ( $fldType == 1114 ) { ## timestamp
				$field->Type('timestamp');
      } elsif ( $fldType == 9 ) {
				$field->Type('date');
			} else {
				$field->Type($fldType);
      }
    }else{
      if ($fldType==-6){ ## tinyint
				$field->Type('tinyint');
      } elsif( $fldType == 4 ){ ## int
				$field->Type('int');
      } elsif ( $fldType == 5 ) { ## smallint
				$field->Type('smallint');
			} elsif ( $fldType == -5 ) {    ## bigint
				$field->Type('bigint');
			} elsif ( $fldType == 11 ) { ## timestamp
				$field->Type('timestamp');
			} elsif ( $fldType == 9 ) { ## date
				$field->Type('date');
			} elsif ( $fldType == 12 ) { ## varchar
				$field->Type('varchar');
      } elsif ( $fldType == 1 ) { ## varchar
				$field->Type('varchar');
      } elsif ( $fldType == -1){ ## blob
				$field->Type('blob');
      } else {
				$field->Type($fldType);
				if ($::DEBUG){
					print STDERR "Found unknown type of field($fldName) : $fldType\n"
				}
      }
    }
    $this->{'fields'}->{$fldName} = $field;
    push @{$this->{'fldnames'}},$fldName;
  }
  return $fldNumb;
}

sub toText {
  my $this=shift;
  my $prefix = shift||'';
  my ($key,$value,$chunk,$cnt);
  my $ret='';
  while ( ($key,$value) = each %{$this}){
    if ( ref($value) =~ /HASH/ ) {
      $ret .= "$prefix$key\t(hash):\n";
      while ( ($key,$chunk) = each %{$value} ) {
				$ret .= $prefix."{$key}\t:\n";
				if ( ref($chunk) =~ /Field/ ) {
					$ret .= $chunk->toText("\t");
				} else {
					$ret .= "$prefix\t$key\t=\t$chunk\n";
				}
      }
    } elsif ( ref($value)=~/ARRAY/ ) {
      $cnt = @{$value};
      $ret .= "$prefix$key\t(array $cnt):\n";
      $cnt = 0;
      foreach $chunk ( @{$value} ) {
				$ret .= $prefix."\t[$cnt]\t=\t$chunk\n";
				$cnt++;
      }
    } else {
      $ret .= "$prefix$key\t=\t$value\n";
    }
  }
  return $ret;
}

1;
