<?
include_once('data_source/class.SQLQuery.php');

function checkMailList( $maillist, $domain_id=NULL, $user=NULL ){

	global $PHP_AUTH_USER;

	if( !$maillist ){
		return array( NULL, ltext('ml_err_emptymaillist') );
	}

	$loUser = new SQLString( $user?$user:$PHP_AUTH_USER );

	if(is_numeric($maillist)){

		$lcSQL = "SELECT ml.name FROM maillist ml, domains d ".
			" WHERE ml.domain_id=d.id AND d.kunde=? AND ml.id=? ";

		if( is_numeric( $domain_id ) ) {
			$lcSQL .= " AND ml.domain_id=?";
			$id = safe_query2( $lcSQL, 
												 $loUser, 
												 new SQLInteger( $maillist ), 
												 new SQLInteger( $domain_id ) 
												 );
		} else {
			$id = safe_query2( $lcSQL, 
												 new SQLInteger($maillist), 
												 $loUser );
		}
		if( db_num_rows($id) ){
			list($name) = db_fetch_array($id,'N');
			return array($name, NULL); 
		}
	}

	if( !preg_match( '/^[a-z0-9_-]+$/', $maillist ) ) {
		return array( NULL, ltext('ml_err_invalidmaillist', array($maillist) ) );		
	}

	$maillist = strtolower($maillist);


	$lcSQL = "SELECT COUNT(*) FROM maillist ml, domains d ".
		"WHERE ml.domain_id=d.id AND d.kunde=? AND ml.name=? ";
	if(is_numeric($domain_id)){
		$lcSQL .= " AND d.id=?";
		$loQuery = new SQLQuery( $lcSQL, array( $loUser, 
																						new SQLValue($maillist,'C'),
																						new SQLValue($domain_id,'N') ) );
	}else{
		$loQuery = new SQLQuery( $lcSQL, array( $loUser, 
																						new SQLValue($maillist,'C') ) );
	}

	$loQuery->doQuery();
	list($cnt) = $loQuery->fetchArray('N');
	if($cnt){
		return  array( NULL, ltext('ml_err_alreadymaillist', array( $maillist ) ) );
	}

	$lcSQL = "SELECT e.prefix, e.domain FROM email e, domains d ".
		" WHERE e.domain = d.domain ".
		" AND d.kunde=? AND e.prefix LIKE '?%'";

	if( is_numeric( $domain_id ) ) {
		$lcSQL .= "AND d.id=?";
		$loQuery->Parameters( array( $loUser, 
																 new SQLValue($maillist,'S'),
																 new SQLValue($domain_id,'N') 
																 )
													);

	}else{
		$loQuery->Parameters( array( $loUser, 
																 new SQLValue($maillist,'S') 
																 )
													);
	}
	$loQuery->Query( $lcSQL ); 
	$loQuery->doQuery();
	while ( list($prefix,$domain) = $loQuery->fetchArray('N') ) {

		if( $maillist == $prefix ){
			return array( NULL, ltext('ml_err_emailconflict', array($prefix.'@'.$domain) ) );
		}

		if( preg_match('/^'.$maillist.'-(owner|approval|outgoiung|archive|request)$/',
									 $prefix ) ){
			return array(NULL, ltext('ml_err_emailconflict', array($prefix.'@'.$domain) ) );
		}
	}

	return array( $maillist, NULL );

}

function canAddMaillist( $lcUser = NULL ){

	global $PHP_AUTH_USER;

	$loUser = $user?new SQLValue($user,'C'): new SQLValue($PHP_AUTH_USER,'C');
	$loQuery = new SQLQuery( "SELECT maxmaillist FROM kunden WHERE kunde=?", $loUser );
	$loQuery->doQuery();
	list($max) = $loQuery->fetchArray('N');
	if($max != -1){
		$loQuery->Query( "SELECT COUNT(name) FROM maillist ml, domains d ".
									 " WHERE ml.domain_id=d.id AND d.kunde=?" );
		$loQuery->doQuery();
		list($cnt) = $loQuery->fetchArray('N');
		if( $cnt>=$max){
			return array(FALSE,ltext('ml_havend', $cnt, $max ));
		}
	}
	return array(TRUE,NULL);
}

function checkOwnerEmail($owner){
	if(!preg_match('/^[\w.-]+\@(?:[\w-]+\.)+\w+$/', $owner))		{
		return array( NULL, ltext('ml_err_invalidemail'));
	}

	return array($owner, NULL);	
}


function checkMailListPassword($p1, $p2 )	{
	if (!( $p1 && $p2) ) {
		return array( NULL, ltext('ml_err_emptypass'));
	}

	if ( (substr_count($p1, " ")) || (substr_count($p2, " ")) ) {
		return array( NULL, ltext('ml_err_invalidpassword'));
	}
	
	if ($p1 != $p2) {
		return array( NULL, ltext('ml_err_passnotmatch'));
	}
	
	return array($p1,NULL);
}

function checkDomainId( $domain_id, $user = NULL ){

	global $PHP_AUTH_USER;

	$loUser = $user?new SQLValue($user,'C'): new SQLValue($PHP_AUTH_USER,'C');
	$lcQuery = "SELECT id,domain FROM domains WHERE kunde=?";
	if(is_numeric($domain_id)){
		$lcQuery .= " AND id=?";
		$loDomain = new SQLValue( $domain_id,'N');
	}else{
		$lcQuery .= " AND domain=?";
		$loDomain = new SQLValue( $domain_id,'C');
	}
	$loQuery = new SQLQuery( $lcQuery, array( $loUser, $loDomain ));
	$loQuery->doQuery();

	if($loQuery->getRowCount()){
		list($domain_id, $domain) = $loQuery->fetchArray('N');

/*
		echo "checkDomainId : domain_id: $domain_id<br>\n";
*/
		return array($domain_id, $domain );

	}else{
		return array(NULL,ltext('ml_domain_not_found'));
	}
}

?>
